#include <Agents/Agent.hpp>
#include <Environment/PictureEnvironment.hpp>

RepaintAgent::RepaintAgent(PictureEnvironment* iEnvironment, int iDotSize, int iMoveSize) :
	Agent(iEnvironment, iDotSize, iMoveSize)
{
	int vWidth = fEnvironment->getWidth();
	int vHeight = fEnvironment->getHeight();

	fXPos = (((double)rand() / RAND_MAX) * vWidth);
	fYPos = (((double)rand() / RAND_MAX) * vHeight);

	fColor = sf::Color(fEnvironment->getCell(fXPos, fYPos));

	//Initalize list of possible movement vectors
	fMovements = std::vector<std::vector<int>>();
	fValidMovementIndexes = std::vector<int>();
	fCurrentMovement = std::vector<int>();
	for (int i = -fMoveSize; i <= fMoveSize; i++)
	{
		for (int j = -fMoveSize; j <= fMoveSize; j++)
		{
			fMovements.push_back({ i, j });
		}
	}
}

void RepaintAgent::move()
{
	fValidMovementIndexes.clear();
	int tolerance = 5;
	for (int i = 0; i < static_cast<int>(fMovements.size()); i++)
	{
		sf::Uint32 va = fEnvironment->getCellOriginal(fXPos + fMovements.at(i).at(0), fYPos + fMovements.at(i).at(1));
		sf::Uint32 vb = (int)fColor.toInteger();
		//std::cout << std::hex << va << " : " << std::hex << vb << "\n";
		//std::cout << std::hex << (va << 8 >> 24) << " : " << (vb << 8 >> 24) << "\n";
		//std::cout << abs((int)(va << 8 >> 24) - (int)(vb << 8 >> 24)) << "\n\n";
		bool vcomp =
			abs((int)(va >> 24) - (int)(vb >> 24)) <= tolerance &&			 //R
			abs((int)(va << 8 >> 24) - (int)(vb << 8 >> 24)) <= tolerance && //G
			abs((int)(va << 16 >> 24) - (int)(vb << 16 >> 24)) <= tolerance; //B

		//if ((double)std::min(va, vb) / std::max(va, vb) > 0.7)
		if (vcomp)
		{
			//std::cout << vcomp << "\n";
			//std::string j;
			//std::cin >> j;
			fValidMovementIndexes.push_back(i);
		}
	}
	if (fValidMovementIndexes.size() > 0)
	{
		int vRandIndex = rand() % static_cast<int>(fValidMovementIndexes.size());
		fCurrentMovement = fMovements.at(fValidMovementIndexes.at(vRandIndex));
		fXPos += fCurrentMovement.at(0);
		fYPos += fCurrentMovement.at(1);
	}
}

void RepaintAgent::paint()
{
	int vHalfBrush = fDotSize / 2;
	int vXLoopIndex = fXPos - vHalfBrush;
	int vYLoopIndex = fYPos - vHalfBrush;
	while (vXLoopIndex < (fXPos + vHalfBrush))
	{
		while (vYLoopIndex < fYPos + vHalfBrush)
		{
			fEnvironment->paint(vXLoopIndex, vYLoopIndex, fColor);
			vYLoopIndex++;
		}
		vYLoopIndex = fYPos - vHalfBrush;
		vXLoopIndex++;
	}
}

void RepaintAgent::loop()
{
	move();
	this->paint();
}
