#include "Platform/Platform.hpp"
#include <Agents/Agent.hpp>
#include <Agents/Perception/Perception.hpp>
#include <Environment/PictureEnvironment.hpp>

const sf::Vector2f DEFAUMT_SIMZE = sf::Vector2f(800, 600);

PictureEnvironment::PictureEnvironment()
{
}

PictureEnvironment::PictureEnvironment(std::string& iFilePath, sf::RenderWindow* iWindow, util::Platform* iPlatform) :
	fSize(iWindow->getSize())
{

	fFilePath = iFilePath;
	//sf::Texture fLoadedImage;
	if (!fTextureImage.loadFromFile(fFilePath))
	{
		std::cout << "\n COULDNT LOAD PICTURE\n";
	}
	fImage = fTextureImage.copyToImage();
	fOriginalImage = fTextureImage.copyToImage();

	//memccpy(&fFilePath, &iFilePath, *iFilePath., 10);
	fPlatform = iPlatform;
	fWindow = iWindow;
	float screenScalingFactor = fPlatform->getScreenScalingFactor(fWindow->getSystemHandle());
	fWindow->create(sf::VideoMode(800.0f * screenScalingFactor, 600.0f * screenScalingFactor), "SFML works!");
	fPlatform->setIcon(fWindow->getSystemHandle());
	fInitialSize.x = 800.0f * screenScalingFactor;
	fInitialSize.y = 600.0f * screenScalingFactor;
	//fShape = sf::RectangleShape(fSize);
	fShape = sf::RectangleShape(DEFAUMT_SIMZE);
	fAgents = std::vector<std::unique_ptr<Agent>>();
	fShape.setFillColor(sf::Color::White);

	fShape.setTexture(&fTextureImage);
}

void PictureEnvironment::getPerception(Perception* ioPercept, int iX, int iY)
{
	int vPerceptHalfWidth = ioPercept->fWidth / 2;
	int vPerceptHalfHeight = ioPercept->fHeight / 2;
	int vLoopX = iX - vPerceptHalfWidth;
	int vLoopY = iY - vPerceptHalfHeight;
	while (vLoopX < iX + vPerceptHalfWidth)
	{
		vLoopX++;
		while (vLoopY < iY + vPerceptHalfHeight)
		{
			if (vLoopX > 0 && vLoopX < getWidth() && vLoopY > 0 && vLoopY < getHeight())
			{
				ioPercept->setXY(
					fImage.getPixel(vLoopX, vLoopY).toInteger(),
					1,
					1);
			}
			vLoopY++;
		}
		vLoopY = iX - vPerceptHalfHeight;
	}
}

bool PictureEnvironment::loop()
{
	if (fWindow->isOpen())
	{
		while (fWindow->pollEvent(fEvent))
		{
			if (fEvent.type == sf::Event::Closed)
				fWindow->close();
		}
		// fAgentsIterator = fAgents.begin();
		std::vector<std::unique_ptr<Agent>>::iterator vAgentsIterator = fAgents.begin();
		while (vAgentsIterator != fAgents.end())
		{
			//std::cout << "loop" << vAgentsIterator->fColor->toInteger() << " at " << &vAgentsIterator << "\n";
			vAgentsIterator->get()->loop();
			vAgentsIterator++;
		}
		fTextureImage.loadFromImage(fImage);
		fShape.setTexture(&fTextureImage);
		fWindow->clear();
		fWindow->draw(fShape);
		fWindow->display();
	}
	else
	{
		std::cout << "\nBah on ferme";
		return false;
	}
	return true;
}

int PictureEnvironment::getHeight()
{
	return fImage.getSize().y;
}

int PictureEnvironment::getWidth()
{
	return fImage.getSize().x;
}

sf::Uint32 PictureEnvironment::getCell(int iX, int iY)
{
	bool doable = iX >= 0 && iX < getWidth() && iY >= 0 && iY < getHeight();
	uint32_t v_Ret = 0;
	if (doable)
	{
		v_Ret = fImage.getPixel(iX, iY).toInteger();
	}
	return v_Ret;
}

sf::Uint32 PictureEnvironment::getCellOriginal(int iX, int iY)
{
	bool doable = iX >= 0 && iX < getWidth() && iY >= 0 && iY < getHeight();
	uint32_t v_Ret = 0;
	if (doable)
	{
		v_Ret = fOriginalImage.getPixel(iX, iY).toInteger();
	}
	return v_Ret;
}

void PictureEnvironment::makeAgents(int iNumber, int iDotSize, int iMoveSize)
{
	for (int i = 0; i < iNumber; i++)
	{
		fAgents.push_back(std::make_unique<Agent>(this, iDotSize, iMoveSize));
	}
}

void PictureEnvironment::makeRepaintAgents(int iNumber, int iDotSize, int iMoveSize)
{
	for (int i = 0; i < iNumber; i++)
	{
		fAgents.push_back(std::make_unique<RepaintAgent>(this, iDotSize, iMoveSize));
	}
}

void PictureEnvironment::makePullPaintAgents(int iNumber, int iDotSize, int iMoveSize)
{
	for (int i = 0; i < iNumber; i++)
	{
		fAgents.push_back(std::make_unique<PullPaintAgent>(this, iDotSize, iMoveSize));
	}
}

void PictureEnvironment::displayLockInput()
{
	fWindow->create(sf::VideoMode(fInitialSize.x, fInitialSize.y), "Last peek");
	fWindow->clear();
	fWindow->draw(fShape);
	fWindow->display();
	int i;
	std::cin >> i;
}

bool PictureEnvironment::paint(int iX, int iY, const sf::Color iColour)
{

	bool doable = iX > 0 && iX < getWidth() && iY > 0 && iY < getHeight();
	if (doable)
	{
		fImage.setPixel(iX, iY, iColour);
	}
	return doable;
}